var MenuUX = MenuUX || {};

// startup of the js layer
$(document).ready(function(){
  MenuUX.parseMenu();
  MenuUX.modifyFieldset();
  MenuUX.checkForErrors();
});

MenuUX.bindOptionsEventHandlers = function() {
  $('.menuux-add-child').click(MenuUX.childClicked);
  $('.menuux-link-inner').click(function() {
    $(this).children('.menuux-add-child').click();
  });
  $('.menuux-add-below').click(MenuUX.belowClicked);
  $('.menuux-add-above').click(MenuUX.aboveClicked);
};

MenuUX.onMenuChanged = function() {
  // show hover menu when an item is clicked
  $('.menuux-expandable > .menuux-item').click(MenuUX.expanderClicked);
};

MenuUX.menuChanged = function(event) {
  // if form field exists, store it's value
  if ($('#menuux-link-input').length) {
    $('#edit-menu-link-title').val($('#menuux-link-input').val());
  }
  // catch errors (and default selection)
  if (typeof(MenuUX.struct[$(this).val()]) !== 'undefined') {
    // catch empty menus
    if (MenuUX.struct[$(this).val()].children.length == 0) {
      $('#menuux-menus-wrapper').html(Drupal.theme('menuuxEmptyBrowser'));
    }
    else {
      $('#menuux-menus-wrapper').html(Drupal.theme('menuuxBrowser', MenuUX.struct[$(this).val()].children, 0));
    }
    MenuUX.bindOptionsEventHandlers();
    if ($('#edit-menu-delete').length > 0) {
      $('#edit-menu-delete').attr('checked', false);
    }
    // if switching back to a menu, refresh data
    if (MenuUX.isActiveMenu($(this).val())) {
      MenuUX.loadDefault();
    }
  }
  else if ($(this).val() == 'menuux-none-selected') {
    // clear any menu selection if none-selected is selected
    $('#menuux-menus-wrapper').html('');
    if ($('#edit-menu-delete').length > 0) {
      $('#edit-menu-delete').attr('checked', true);
      $('#menuux-menus-wrapper').html('<p>' + Drupal.t('Previous menu choice will be deleted.') + '</p>');
    }
    $('#edit-menu-link-title, #edit-menu-menuux-above').val('');
  }
  MenuUX.onMenuChanged();
}

MenuUX.isActiveMenu = function(menuName) {
  var parent = $('#edit-menu-parent').val().split(':');
  return menuName == parent[0];
}

// startup of the menu browser
MenuUX.modifyFieldset = function() {
  MenuUX.$menuFieldset = $('#edit-menu-link-title').parents('.fieldset-wrapper, .vertical-tabs-menu');
  $('.menuux-hide-me', MenuUX.$menuFieldset).each(function() {
    $('#' + $(this).attr('id') + '-wrapper').css('display', 'none');
  });
  MenuUX.$menuFieldset.prepend(Drupal.theme('menuuxMenu', MenuUX.struct));

  // get and set default menu
  var parent = $('#edit-menu-parent').val().split(':');
  $('#menuux-parent').change(MenuUX.menuChanged).val(parent[0]).trigger('change');
}

// loading of any default values for editing or when submit fails
MenuUX.loadDefault = function() {
  var above = $('#edit-menu-menuux-above').val();
  var parent = $('#edit-menu-parent').val().split(':');
  var parentMlid = parent[1];

  // if no above, but a parent exist = child
  if (above == 'top') {
    // find out if the parent has children or not
    if (parentMlid > 0 && !MenuUX.index[$('#menuux-parent').val() + ':' + parentMlid].hasChildren()) {
      var $parent = $('.menuux-link[data-mlid='  + parentMlid + ']');
      MenuUX.childClicked.call($(' > .menuux-link-inner .menuux-add-child', $parent));
      $parent.removeClass('menuux-collapsed')
      .parents('.menuux-collapsed')
      .removeClass('menuux-collapsed');
    }
    else {
      if (parentMlid == 0) {
        MenuUX.aboveClicked.call($('.menuux-link:first > .menuux-add-above'));
      }
      else {
        MenuUX.aboveClicked.call($('.menuux-link:first > .menuux-add-above', $('.menuux-link[data-mlid='  + parentMlid + ']')));
      }
    }
  }

  else if (above > 0) {
    var $above = $('.menuux-link[data-mlid=' + above + ']');
    MenuUX.belowClicked.call($(' > .menuux-add-below', $above));
    $above.removeClass('menuux-collapsed')
    .parents('.menuux-collapsed')
    .removeClass('menuux-collapsed');
  }
}

MenuUX.checkForErrors = function() {
  if ($('#menuux-title-error').length) {
    $('#menuux-link-input').addClass('menuux-form-error').val('');
  }
}

// expands an expandable menu
MenuUX.expanderClicked = function(event) {
  var $link = $(this).parent();
  if ($link.hasClass('menuux-collapsed')) {
    $link.removeClass('menuux-collapsed')
  }
  else {
    $link.addClass('menuux-collapsed')
  }
};

// finds the correct target for clicks or onload events
MenuUX.findTarget = function($target) {
  if ($target.is('.menuux-link, .menuux-level-0')) {
    return $target;
  }
  else {
    return $target.parents('.menuux-link').eq(0);
  }
}

MenuUX.belowClicked = function(event) {
  MenuUX.siblingClicked.call(this, event, 'below');
  MenuUX.newMenuItemMoved.call(this);
};

MenuUX.aboveClicked = function(event) {
  MenuUX.siblingClicked.call(this, event, 'above');
  MenuUX.newMenuItemMoved.call(this);
  $('#menuux-empty').html(Drupal.t('This is the only content in this menu.'));
};

MenuUX.siblingClicked = function(event, type) {
  var $newMenuField = MenuUX.newMenuField.call(this, event);
  var $sibling = MenuUX.findTarget($(this));
  if (type == 'above') {
    $newMenuField.insertBefore($sibling);
  }
  else {
    $newMenuField.insertAfter($sibling);
  }
  var parent = $sibling.parents('.menuux-link').eq(0);
  $sibling.parents('.menuux-collapsed').removeClass('menuux-collapsed')
  if (parent.length == 0) {
    // set parent to root of selected menu
    $('#edit-menu-parent').val($('#menuux-parent').val()+":0");
  } else {
    $('#edit-menu-parent').val($('#menuux-parent').val()+":"+parent.attr('data-mlid'));
  }
  $('#edit-menu-menuux-above').val(type == 'above' ? 'top' : $sibling.attr('data-mlid'));
};

MenuUX.childClicked = function(event) {
  var $newMenuField = MenuUX.newMenuField.call(this, event);
  var $parent = MenuUX.findTarget($(this));
  // create an ul and inject
  $('<ul class="menuux-link menuux-link-new"></ul>')
    .append($newMenuField)
    .insertAfter($parent.find('.menuux-link-inner').eq(0));
  $('#edit-menu-menuux-above').val('top');
  // expand if adding as a child
  $parent.removeClass('menuux-collapsed');
  // set value of original form's parent to selected parent
  $('#edit-menu-parent').val($('#menuux-parent').val() + ":" + $parent.attr('data-mlid'));
  MenuUX.newMenuItemMoved.call(this);
};

MenuUX.newMenuItemMoved = function() {
  $('.menuux-hidden').removeClass('menuux-hidden');
  var $this = $(this);
  if ($this.hasClass('menuux-add-below') || $this.hasClass('menuux-add-above') || $this.hasClass('menuux-add-child')) {
    $(this).addClass('menuux-hidden');
  }
  $('.menuux-add-content').text(Drupal.t('Move here'));
  $('#menuux-link-input:not(.menuux-processed)').addClass('menuux-processed').change(function() {
    $('#edit-menu-link-title').val($('#menuux-link-input').val());
  });
  $('#edit-menu-link-title').val($('#menuux-link-input').val());
};

// creates a new menu field
MenuUX.newMenuField = function(event) {
  if (event) {
    event.stopPropagation();
    event.preventDefault();
  }
  // if a new field has already been created, refer to that field so it
  // is moved instead of creating a new one
  if ($('#menuux-new').length > 0) {
    return $('#menuux-new');
  }
  var output = '<li id="menuux-new" class="menuux-link"><div class="menuux-link-inner clear-block"><div class="menuux-link-title">'
    + MenuUX.createLinkField() + '</div></div></li>';
  return $(output);
}

// creates an input form for newMenuField
MenuUX.createLinkField = function() {
  var extra = ' placeholder="' + Drupal.t('Write title here') + '"';
  var title = $('#edit-menu-link-title').val();
  if (title.length == 0 && $('#edit-title').length) {
    title = $('#edit-title').val();
  }
  if (title.length) {
    extra = ' value="' + title + '"';
  }
  var formClass = 'form-item';
  if ($('#edit-menu-menuux-has-children').val() == 1) {
    formClass += ' menuux-has-children';
  }
  return Drupal.theme('menuuxLinkField', formClass, extra);
};

// parses the menu
MenuUX.parseMenu = function() {
  MenuUX.struct = {};
  MenuUX.index = {};
  var lasts = [];
  $('#edit-menu-parent').children('option').each(function() {
    var link = new MenuUXLink($(this).text(), $(this).attr('value'));
    if (link.isMenu) {
      MenuUX.struct[link.menuId] = link;
    }
    else {
      lasts[link.level - 1].addChild(link);
    }
    lasts[link.level] = link;
    MenuUX.index[$(this).attr('value')] = link;
  });
};

// constructor for menu links
MenuUXLink = function(text, value) {
  this.isMenu = text.slice(0, 1) == '<';
  this.level = 0;
  this.children = [];
  if (this.isMenu) {
    this.menuName = text.slice(1, text.length - 1);
  }
  else {
    while (true) {
      if (text.slice(this.level * 2, this.level * 2 + 2) == '--') {
        this.level++;
      }
      else {
        break;
      }
    }
    this.title = text.slice(this.level * 2 + 1, text.length)
  }
  var ids = value.split(':', 2);
  this.menuId = ids[0];
  this.mlid = ids[1];
  this.addChild = function(child) {
    this.children.push(child);
  }
  this.hasChildren = function() {
    return this.children.length > 0;
  }
};

// main wrapper theme
Drupal.theme.prototype.menuuxMenu = function(struct) {
  var output = '<div id="menuux-browser">'
    + '<div id="menuux-parent-wrapper" class="form-item">'
    + '<label for="menuux-parent">' + Drupal.t('Menu') +': </label>'
    + '<select id="menuux-parent" class="form-select">'
    + '<option value="menuux-none-selected">- ' + Drupal.t('None selected') + ' -</option>';
  for (var i in struct) {
    output += '<option value="' + struct[i].menuId + '">' + struct[i].menuName + '</option>';
  }
  output += '</select><div id="menuux-menus-wrapper"></div></div>'
    + '<p class="tips">'
    + Drupal.t('To remove this content from the menu choose %option in the menu selector.', {'%option': '- ' + Drupal.t('None selected') + ' -'})
    + '</p></div>';
  return output;
};

// menu browser theme
Drupal.theme.prototype.menuuxBrowser = function(struct, level) {
  var output = '<ul class="menuux menuux-level-' + level + '">';
  var first = true;
  for (var i in struct) {
    var link = struct[i];
    var liClass = link.hasChildren() ? ' menuux-collapsed menuux-expandable' : '';
    output += '<li class="menuux-link clear-block' + liClass + '" data-mlid="' + link['mlid'] + '">';
    if (first) {
      output += '<div class="menuux-add-above"><div class="menuux-add-content">' + Drupal.t('Add here') + '</div></div>';
      first = false;
    }
    output += '<div class="menuux-link-inner clear-block menuux-item">';
    var children = link.hasChildren() ? Drupal.theme('menuuxBrowser', link.children, level + 1) : '';
    var addChild = link.hasChildren() ? '' : '<div class="menuux-add-child">' + Drupal.t('Add child') + '</div>';
    output += '<div class="menuux-link-title">' + link.title + '</div>' + addChild + '</div>' + children +
      '<div class="menuux-add-below"><div class="menuux-add-content">' + Drupal.t('Add here') + '</div></div>';
    output += '</li>';
  }
  output += '</ul>';
  return output;
};

Drupal.theme.prototype.menuuxEmptyBrowser = function() {
  return '<ul class="menuux menuux-level-0"><li class="menuux-link clear-block">'
    + '<div class="menuux-add-above"><div class="menuux-add-content">'
    + Drupal.t('Add here')
    + '</div></div></li></ul>'
    + '<div id="menuux-empty" class="description">' + Drupal.t('This menu is empty. Click the "Add here" link above to add your node to this menu.') + '</div>';
};

Drupal.theme.prototype.menuuxLinkField = function(formClass, extra) {
  return '<div class="' + formClass + '"><input class="form-text" id="menuux-link-input" type="text" size="60" maxlength="255"' + extra + '/></div>';
}